"""Examples to using decorators."""


def print_info(function):
    """Print information about the given function to execute.

    Args:
        function (function): Function to execute.

    Returns:
        function: Wrapped function where we print information about the used
            function and the passed in arguments before and after invoking the
            function.

    """

    def wrapper(*args):
        """Wrap the function to execute.

        Args:
            args (multiple): Arguments that were passed into the function.

        """
        information = [function]
        for arg in args:
            information.append(arg)
        print "Executing: {}".format(information)
        result = function(*args)
        print "Finished executing {}".format(information)
        return result

    return wrapper


@print_info
def hello():
    """Print hello to the console."""
    print "hello"


@print_info
def repeat(word, count):
    """Repeat the given word count times.
    
    Args:
        word (str): The word to repeat.
        count (int): The number of repetitions to apply.

    """
    return word * count


def ensure_integer_arguments(function):
    """Ensure that all arguments are integers prior to invoking a function.

    Args:
        function (function): Function to execute.

    """

    def wrapper(*args):
        """Check that all args are integers prior to executing.

        Args:
            args (multiple): Arguments to check for type.

        raises:
            ValueError: When any passed in argument is not of integer type.

        """
        for index, arg in enumerate(args):
            if not isinstance(arg, int):
                error = ("{}: Invalid non integer value for {}. "
                         "argument: {}.".format(function.__name__, index + 1,
                                                arg))
                raise ValueError(error)

        return function(*args)

    return wrapper


@ensure_integer_arguments
def multiply(a):
    """Multiply the given number with itself.

    Args:
        a (int): The number to use for calculation.

    Returns:
        int: The number multiplied with itself.

    """
    return  a * a


@ensure_integer_arguments
def add(a, b):
    """Get the sum of the given numbers.

    Args:
        a (int): The first number to use for calculation.
        b (int): The second number to use for calculation.

    Returns:
        int: The sum of a and b.

    """
    return  a + b
